#include <stdio.h>
#include <string.h>
#include <windows.h>

#define scale 0.2
#define X_SC (0.0338)
#define Y_SC (0.0338)

static	HWND	wnd;
static	HDC		dc;
static	int		cur_x, cur_y, last_x, last_y;
static	int		screen_height, screen_width;
static	char	dotchar = 0;
static	char	coords_string[256];
static	int		inc_mode = 0;
static	FILE	*out = NULL;

static void
move_to(int x,int y)
{
	static POINT stub;
	MoveToEx(dc, x, y, &stub);
}

static void
print(int x,int y,char *str)
{
	TextOut(dc, x, y, str, strlen(str));
}

static void
print_coords(void)
{
	char buf[256], *ptr;
	sprintf(buf,"%d          ",cur_x);
	print(10,10,buf);
	sprintf(buf,"%d          ",cur_y);
	print(10,30,buf);
	sprintf(buf,"%s        ",coords_string);
	for(ptr = buf;*ptr;ptr++)
		if(*ptr < 32)
			*ptr = 32;
	print(10, 50, buf);
}

static void
redraw(void)
{
	int hw = screen_width >> 1, hh = screen_height >> 1;
	PatBlt(dc, 0, 0, screen_width, screen_height, WHITENESS);
	move_to(0, hh);
	LineTo(dc, screen_width-1, hh);
	move_to(hw,0);
	LineTo(dc, hw, screen_height - 1);
}

static void
line(void)
{
	int hw = screen_width >> 1, hh = screen_height >> 1;
	move_to(hw, hh);
	LineTo(dc,
		hw + (int)(scale * (cur_x - last_x)),
		hh - (int)(scale * (cur_y - last_y)));
}

static void
dump_time()
{
	SYSTEMTIME sm;
	GetLocalTime(&sm);
	fprintf(out,"YEAR;%u;MONTH;%u;DAY;%u;HOUR;%u;MIN;%u;SEC;%u;MS;%u;",
		sm.wYear, sm.wMonth, sm.wDay, sm.wHour, sm.wMinute, sm.wSecond, sm.wMilliseconds);
}

static void
next_point(void)
{
	sprintf(coords_string,"%g;%g",cur_x * X_SC,cur_y * Y_SC);
	if(dotchar)
	{
		char *ptr;
		for(ptr = coords_string;*ptr;ptr++)
			if(*ptr == '.')
				*ptr = dotchar;
	}
	dump_time();
	fprintf(out,"%s\n",coords_string);
	last_x = cur_x;
	last_y = cur_y;
	redraw();
	print_coords();
}

static void
reset(void)
{
	last_x = last_y = cur_x = cur_y = 0;
	redraw();
	print_coords();
}

LRESULT CALLBACK
WindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	PAINTSTRUCT ps;
	int x, y, cx, cy;
	cx = screen_width >> 1;
	cy = screen_height >> 1;
	switch(uMsg)
	{
		case WM_MOUSEMOVE:
			x = LOWORD(lParam);
			y = HIWORD(lParam);
			if(x & 0x8000)
				x |= 0xFFFF0000;
			if(y & 0x8000)
				y |= 0xFFFF0000;
           	cx = screen_width >> 1;
            cy = screen_height >> 1;
			/* Avoid recursion */
			if(x == cx && y == cy)
				break;
           	/* Store deltas */
			cur_x += x - cx;
			cur_y -= y - cy;
			SetCursorPos(cx, cy);
			line();
			print_coords();
			if(!inc_mode)
				break;
			/* fall throw */
		case WM_LBUTTONDOWN:
			next_point();
			break;
		case WM_RBUTTONDOWN:
			reset();
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			return 0;
		case WM_KEYDOWN:
			switch(wParam)
			{
				case VK_ESCAPE:
					PostQuitMessage(0);
					break;
				case VK_SPACE:
					next_point();
					break;
				case VK_RETURN:
					reset();
					break;
				case VK_BACK:
					cur_x = last_x;
					cur_y = last_y;
					redraw();
					print_coords();
					break;
				default:
					break;
			}
			break;
		default:
			return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	return 0;
}

static void
usage(char *self)
{
	static char helpmsg[] =
	"%s ver. 0.2. Vladimir A Smirnov (kkatarn@sleepgate.ru), 2002-2012.\n" \
	"Simple tool for manual discretization of 2d plots.\n" \
	"Glue a thin wire to mouse. Start app. Move the mouse along graph,\n" \
	"pressing space or lbutton. Press ESC when done.\n" \
	"Usage: %s [--inc] [--outfile csvfname] [--dotchar decdelim] [--help]\n" \
	"Bugs: operational under FreeBSD+wine, but not under VirtualBox+WinXP.\n";
	printf(helpmsg, self, self);
}

int main(int argc, char *argv[])
{
	int i;
	MSG msg;
	char *outname = NULL;
	WNDCLASS wc = {
		0,
		WindowProc,
		0,
		0,
		GetModuleHandle(NULL),
		LoadIcon(NULL, IDI_APPLICATION),
		LoadCursor(NULL, IDC_ARROW),
		GetStockObject(WHITE_BRUSH),
		NULL,
		"DigiClass" };
	for(i = 1;i < argc;i++)
	{
		if(!strcmp(argv[i], "--inc"))
			inc_mode = 1;
		else if(!strcmp(argv[i], "--outfile"))
		{
			if(++i == argc)
			{
				fprintf(stderr,"%s(): not enough arguments.\n", argv[0]);
				return -1;
			}
			outname = argv[i];
		}
		else if(!strcmp(argv[i], "--dotchar"))
		{
			if(++i == argc)
			{
				fprintf(stderr,"%s(): not enough arguments.\n", argv[0]);
				return -1;
			}
			if(strlen(argv[i]) > 1)
				fprintf(stderr,
					"%s(): using 1st char of \"%s\" as decimal separator.\n",
					argv[0], argv[i]);
			dotchar = argv[i][0];
		}
		else if(!strcmp(argv[i], "--help"))
		{
			usage(argv[0]);
			return 0;
		}
		else
			fprintf(stderr,"%s(): unknown option \"%s\"\n",
				argv[0], argv[i]);
	}		
	if(outname != NULL)
	{
		out = fopen(outname, "wb");
		if(!out)
			fprintf(stderr,"%s(): unable to open \"%s\" for writing.\n",
				argv[0], outname);
	}
	if(!out)
		out = stdout;
	if(!RegisterClass(&wc))
	{
		fprintf(stderr,"%s(): RegisterClass() failed.\n", argv[0]);
		if(out != stdout)
			fclose(out);
		return -1;
	}
	screen_width = GetSystemMetrics(SM_CXSCREEN);
	screen_height = GetSystemMetrics(SM_CYSCREEN);
	if(screen_width < 320 || screen_height < 200)
	{
		fprintf(stderr,"%s(): GetSystemMetrics() returned funny values.\n",
			argv[0]);
		if(out != stdout)
			fclose(out);
		return -1;
	}
	/* Center cursor */
	SetCursorPos(screen_width >> 1,screen_height >> 1);
	wnd = CreateWindow(
		wc.lpszClassName,
		"Digitizer",
		WS_POPUP,
		0,0,screen_width,screen_height,
		NULL,
		NULL,
		wc.hInstance,
		NULL);
	if(!wnd)
	{
		fprintf(stderr,"%s(): CreateWindow() failed.\n", argv[0]);
		if(out != stdout)
			fclose(out);
		return -1;
	}
	ShowCursor(FALSE);
	ShowWindow(wnd, SW_SHOW);
	UpdateWindow(wnd);
	dc = GetDC(wnd);
	redraw();
	dump_time();
	fprintf(out, "START\n");
	while(GetMessage(&msg,wnd,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	ReleaseDC(wnd, dc);
	ShowCursor(TRUE);
	DestroyWindow(wnd);
	UnregisterClass(wc.lpszClassName, wc.hInstance);
	dump_time();
	fprintf(out, "END\n");
	if(out != stdout)
		fclose(out);
	return 0;
}
